
SYSSTACK    dc.l $001c3bd4	;Stack pointer
FIRSTINST   dc.l $00000654	;First instruction

    ORG    $654	;where the original software starts. Can be changed but never did
START:

;These first few lines are part of the original software and is used for initialisation of the hardware
    MOVE	SR,D0			
	ORI	#$0700,D0		
	MOVE	D0,SR			
	MOVEA.L	#$001C3FD4,A0		
	MOVE.L	A0,USP			
	MOVE	#$0000,BACKCOLOUR		
	MOVE	#$0000,EXT_001E		
	MOVE	#$FFFF,EXT_001F		
	MOVE	#$0000,EXT_0020		
	CLR	WATCHDOG		
	MOVE	#$0000,D0		

;Start of RAM clearing routines
;If this isnt carried out then the screen is unreadable as RAM isnt zeroed on real hardware like it is in MAME
	
	;FG0 video RAM - Dont think a read is possible
	LEA	FG0_VIDEORAM,A0		
	MOVE	#$07FF,D1		
	BSR	CLEARMEM		

	;Sprite RAM
	LEA	SPRITERAM,A0		
	MOVE	#$07FF,D1		
	BSR	CLEARMEM	
	
	;Palette RAM
	LEA PALETTE_RAM,A0
	MOVE 	#$07ff,D1
	BSR CLEARMEM	
	
	;BG0 video RAM
	LEA	BG0_VIDEORAM,A0		
	MOVE	#$03FF,D1		
	BSR	CLEARMEM		
	
	;BG0 video RAM		
	LEA	BG1_VIDEORAM,A0		
	MOVE	#$03FF,D1		
	BSR	CLEARMEM		

	CLR	SCREENFLG	;if this isnt called, part of the screen will still be garbage from startup, not sure what it really is
					
	;more initialisation routines
	MOVE	#$0081,EXT_0014		
	MOVE	#$0050,EXT_0015		
	MOVE	#$00AB,EXT_0016		
	MOVE	#$0078,EXT_0018		
	MOVE	#$00F7,EXT_0019		
	MOVE	#$0001,EXT_001A		

	BSR	INITIALIZE		;screen text setup
	BRA	RUNTEST		

INITIALIZE:		;screen text setup
	CLR	WATCHDOG		;not sure how often the watchdog needs cleared so may have gone overboard on it. MAME doesnt have it recorded
	CLR.L	D0			
	CLR.L	D1			
	MOVE	#$00f0,FORECOLOUR		;palette setup for colour red, [DC: but we splat over this later so it ends up green]

	LEA	WWF_TITLE_STR,A0			;load the address in ROM of the text required
	LEA	TITLE_STR_VRAM,A1			;location on the screen
	MOVE.B	#$17,D1					;text counter. This is the total number of chars in the text starting at 0
	BSR	PRINTTEXT					;subroutine to write all text in the proper places in RAM

	LEA	BG0_VIDEO_STR,A0		
	LEA	BG0_STR_VRAM,A1		
	MOVE.L	#$0000000D,D1		
	BSR	PRINTTEXT		

	LEA	BG1_VIDEO_STR,A0		
	LEA	BG1_STR_VRAM,A1		
	MOVE.L	#$0000000D,D1		
	BSR	PRINTTEXT		

	LEA PALETTE_STR,A0
	LEA PALETTE_STR_VRAM,A1
	MOVE.L  #$0000000B,D1
	BSR PRINTTEXT

	LEA SPRITE_STR,A0
	LEA SPRITE_STR_VRAM,A1
	MOVE.L  #$0000000A,D1
	BSR PRINTTEXT

	LEA	SITENAME_STR,A0		
	LEA	SITE_STR_VRAM,A1		
	MOVE.L	#$00000011,D1		
	BSR	PRINTTEXT		

	RTS						

RUNTEST:
	LEA	BG0_VIDEORAM,A0			;Address of memory to test
	LEA	BG0_RESULT_VRAM,A1		;Address of vram to print result		
	MOVE	#$07FF,D1			;Loop counter of memory to test
	BSR	TEST_MEM				
	LEA	BG0_VIDEORAM,A0			
	MOVE	#$03FF,D1			
	BSR	CLEARMEM				;Clear memory after use

	LEA	BG1_VIDEORAM,A0		
	LEA	BG1_RESULT_VRAM,A1		
	MOVE	#$07FF,D1		
	BSR	TEST_MEM		
	LEA	BG1_VIDEORAM,A0		
	MOVE	#$03FF,D1		
	BSR	CLEARMEM		

	LEA	PALETTE_RAM,A0	
	LEA	PALETTE_RESULT_VRAM,A1				
	MOVE	#$7FFF,D1		
	BSR	TEST_MEM	

	BSR LOAD_PALETTES			;Load palette memory
	BSR LOAD_SPRITES			;Load sprite memory
	
	BRA	INFINITE_LOOP			;enter infinite loop

; Function LOAD_PALETTES
; Load Title screen sprite palette data from Mame RAM dump
LOAD_PALETTES:
	LEA PALETTE2,A0
	LEA PALETTE2_RAM,A1
	MOVE.W #$3FF,D0
PALETTE_INNER:
	MOVE.W (A0)+,D2
	MOVE.W D2,(A1)+
	DBF D0,PALETTE_INNER
	RTS

; Function LOAD_SPRITES
; Load title screen sprite data from Mame RAM dump
LOAD_SPRITES:
	LEA SPRITEBUF,A0	; Source data of sprite data
	LEA SPRITERAM,A1	; Sprite RAM start address
	MOVE #$0FFF,D0		; Sprite RAM copy size
LOAD_SPRITE_LOOP:
	MOVE.W (A0)+,D1
	MOVE.W D1,(A1)+
	DBF D0,LOAD_SPRITE_LOOP
	RTS

; Function TEST_MEM
; A0 = Address of buffer to test
; D1 = Size of buffer
TEST_MEM:
	CLR	WATCHDOG		
	CLR	SCREENFLG		
	MOVE	#$5555,D0	
TEST_MEM_LOOP:
	MOVE 	(A0),D2
	MOVE	D0,(A0)
	MOVE 	(A0),D3
	MOVE 	D2,(A0)+				
	CMP	D3,D0		
	BNE	PRINT_WRITE_FAILED		
	DBF	D1,TEST_MEM_LOOP		
	BSR	PRINT_WRITE_PASSED		
	RTS				

; Function INFINITE_LOOP
; Used to keep the program running but not affecting the screen display.
INFINITE_LOOP:
	NOP				
	CLR	WATCHDOG		
	BRA.S	INFINITE_LOOP	

; Function PRINT_WRITE_PASSED	
PRINT_WRITE_PASSED:
	LEA	PASSED_STR,A0		
	MOVE.L	#$00000005,D1		
	BSR	PRINTTEXT		
	RTS

; Function PRINT_WRITE_FAILED	
; Dump failure message, and print the address of the failure
PRINT_WRITE_FAILED:
	MOVE.L A0,A2
	SUB.L #$1,A2
	CLR	WATCHDOG		
	CLR	SCREENFLG		
	LEA	FAILED_STR,A0		
	MOVE.L	#$00000005,D1		
	BSR	PRINTTEXT
	BSR PRINTADDR	
	BRA	RUNTEST		

; Function CLEARMEM
; A0 = Addres of buffer
; D1 = Length of buffer
CLEARMEM:
	CLR	WATCHDOG		
	CLR.L	(A0)+			
	DBF	D1,CLEARMEM		
	RTS	

; Function PRINTTEXT
; A0 = Address of string
; A1 = Address of Display RAM
; D1 = String Length			
PRINTTEXT:
	MOVE.B	(A0)+,D0		
	SUBI	#$0010,D0	;subtract 0x10 from the char hex code as the char gfx sit further down in address space. Check MAME gfx viewer	
	MOVE	D0,(A1)			
	ADDA	#$0004,A1		
	DBF	D1,PRINTTEXT		
	CLR	WATCHDOG		
	CLR	SCREENFLG		
	RTS	

; Function PRINTADDR 
; Dump an address to the screen in hex
; A2 = Address to dump
; A1 = Address of Display RAM
PRINTADDR:
	MOVE #$0007,D4
	ADDA #$0020,A1
	MOVE.L A2,D5
LOOP:
	MOVE D5,D1
	AND #$000F,D1
	LEA NUM_ARRAY,A3
	ADD D1,A3
	MOVE.B (A3),D1	; lookup hex digit from lookup table
	SUBI #$0010,D1  ; subtract 0x10 from the char hex code as the char gfx sit further down in address space. Check MAME gfx viewer	
	MOVE D1,(A1)
	SUB #$4,A1
	ROR.L #$0004,D5
	DBF D4,LOOP
	RTS

SPRITERAM		EQU	$C2000
BG0_VIDEORAM	EQU	$80000
BG1_VIDEORAM	EQU	$82000
FG0_VIDEORAM	EQU	$C0000
PALETTE_RAM		EQU	$180000
PALETTE2_RAM	EQU $182000

TITLE_STR_VRAM	EQU	$C0724
BG0_STR_VRAM	EQU	$C0914
BG0_RESULT_VRAM	EQU	$C0950
BG1_STR_VRAM	EQU	$C0B14
BG1_RESULT_VRAM	EQU	$C0B50
PALETTE_STR_VRAM EQU $C0D14
PALETTE_RESULT_VRAM EQU $C0D50
SPRITE_STR_VRAM	EQU	$C0F14
SPRITE_RESULT_VRAM EQU	$C0F50
SITE_STR_VRAM	EQU	$C1124

;Initialization parameters
EXT_0014	EQU	$100008
EXT_0015	EQU	$10000C
EXT_0016	EQU	$140006
EXT_0018	EQU	$140010
EXT_0019	EQU	$140012
EXT_001A	EQU	$140014
EXT_001E	EQU	$1C0076
EXT_001F	EQU	$1C007A
EXT_0020	EQU	$1C1680

SCREENFLG	EQU	$140008
WATCHDOG	EQU	$140016
FORECOLOUR	EQU	$18001E
BACKCOLOUR	EQU	$184000 

WWF_TITLE_STR   dc.b    'WWF WRESTLEFEST RAM TEST   SPRITE RAM\n',0
FG0_VIDEO_STR   dc.b    'FG0 VIDEO RAM\n',0
BG0_VIDEO_STR   dc.b    'BG0 VIDEO RAM\n',0
BG1_VIDEO_STR   dc.b    'BG1 VIDEO RAM\n',0
PALETTE_STR		dc.b 	'PALETTE RAM\n',0
SPRITE_STR 		dc.b 	'SPRITE RAM\n',0
SITENAME_STR    dc.b    'WWW.JAMMARCADE.NET',0
PASSED_STR      dc.b    'PASSED',0
FAILED_STR      dc.b    'FAILED',0
NUM_ARRAY		dc.b 	'0123456789ABCDEF'

SPRITEBUF		INCBIN	"sprite.dmp"
PALETTE2		INCBIN	"palette2.dmp"

    END    START        ; last line of source

    